/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.communication.topology;

import java.util.Set;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.microkernel.api.communication.topology.exception.AddSubDomainException;
import org.ow2.petals.microkernel.api.communication.topology.exception.LockAlreadyAcquiredException;
import org.ow2.petals.microkernel.api.communication.topology.exception.RemoveContainerException;
import org.ow2.petals.microkernel.api.communication.topology.exception.RemoveSubDomainException;
import org.ow2.petals.microkernel.api.communication.topology.exception.TopologyException;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * Interface of the component 'Topology'
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public interface TopologyService extends TopologyServiceMBean {

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#TOPOLOGY_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * <p>
     * Loads the current topology from the registry or initializes it into the registry.
     * </p>
     * <p>
     * The initialization is done on the first container startup of the topology from the local topology definition.
     * Otherwise, the local topology definition is replaced by the one available in the registry.
     * </p>
     * <p>
     * Note: the definition of a dynamic topology is maintained and shared through the registry.
     * </p>
     * 
     * @throws LockAlreadyAcquiredException
     *             Unable to acquire the lock about dynamic topology
     * @throws TopologyException
     *             An unknown error occurs loading the topology
     */
    public void loadTopology() throws LockAlreadyAcquiredException, TopologyException;

    /**
     * Add a subdomain to the local topology.
     * 
     * @param addedSudomain
     * @throws AddSubDomainException
     */
    void addSubdomainConfiguration(SubDomainConfiguration addedSubdomain)
            throws AddSubDomainException;

    /**
     * Get the container configuration of the given container name.
     * 
     * @param containerName
     *            The name of the targeted container
     * @return The container configuration
     */
    ContainerConfiguration getContainerConfiguration(String containerName);

    /**
     * Get all the containers where the state is the same than the given one. If
     * the state is null, get all the containers...
     * 
     * @param object
     * @return
     */
    Set<ContainerConfiguration> getContainersConfiguration(ContainerState state);

    Set<ContainerConfiguration> getContainersConfigurationsForLocalSubdomain();

    /**
     * Get the domain configuration.
     * 
     * @return the domain configuration
     */
    DomainConfiguration getDomainConfiguration();

    /**
     * Get all the sub-domain configurations.
     * 
     * @return a set of {@link ContainerConfiguration}
     */
    Set<SubDomainConfiguration> getSubDomainsConfiguration();

    /**
     * Test if the container is started.
     * 
     * @param containerName
     * @return
     */
    boolean isContainerStarted(String containerName);

    /**
     * Remove container configuration.
     * 
     * @param removedContainer
     * @throws RemoveContainerException
     */
    void removeContainerConfiguration(ContainerConfiguration removedContainer)
            throws RemoveContainerException;

    /**
     * Remove sub-domain configuration.
     * 
     * @param removedSubdomain
     * @throws RemoveSubDomainException
     */
    public void removeSubdomainConfiguration(final SubDomainConfiguration removedSubdomain)
            throws RemoveSubDomainException;

    /**
     * Set the given state to the give container.
     * 
     * @param containerName
     * @param state
     *            The {@link ContainerConfiguration.ContainerState}
     */
    void setContainerState(String containerName, ContainerConfiguration.ContainerState state);
}
