/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerAlreadyExistException;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerDoesNotExistException;
import org.ow2.petals.microkernel.api.configuration.exception.InvalidContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.InvalidSubdomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.MissingServerPropertyException;
import org.ow2.petals.microkernel.api.configuration.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoDomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoDomainNameProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoServerPropertiesProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoSubdomainNameProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoTopologyProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainAlreadyExistException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainDoesNotExistException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainUnknownException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.topology.generated.Topology;

/**
 * <p>
 * The interface of the Configuration service that handles the configuration of the local PEtALS container.
 * </p>
 * <p>
 * During the execution of methods {@link #loadConfiguration(Properties, Topology)} or {@link #updateTopology(Topology)}
 * , the other methods are blocked until the end of the configuration and topology loading.
 * </p>
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @author Charles Casadei - EBM WebSourcing
 * @since Petals 2.0
 */
public interface ConfigurationService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.ConfigurationService";

    static final String JAAS_PROPS_FILE = "/jaas.properties";

    static final String PROPERTY_REGISTRY_MODE = "registry.mode";

    static final String STANDALONE = "standalone";

    static final String PEER = "peer";

    static final String MASTER = "master";

    static final String SLAVE = "slave";

    /**
     * Name of the server interface of the Fractal component
     * {@value FractalHelper#CONFIGURATION_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";
    
    /**
     * Get the local container configuration.
     * 
     */
    ContainerConfiguration getContainerConfiguration();

    /**
     * <p>
     * Get the local server properties (ie. content of the file '
     * <code>server.properties</code>' or equivalent), <b>including properties
     * not configured</b> (ie. set to a <code>null</code> value).
     * </p>
     * <p>
     * Note: local server properties are provided as a {@link Map} instead of
     * {@link Properties} to be able to put <code>null</code> values.
     * </p>
     * 
     * 
     */
    Map<String, String> getAllServerProperties();
    
    /**
     * Get the local server properties (ie. content of the file ' <code>server.properties</code>' or equivalent),
     * <b>including only properties configured</b> (ie. set to a non <code>null</code> value).
     * 
     * @return The local server properties. Cannot be <code>null</code>.
     */
    public Properties getServerProperties();

    /**
     * @return The container configuration of remote containers. The local container configuration is returned by
     *         {@link #getContainerConfiguration()}
     */
    public Set<ContainerConfiguration> getContainersConfiguration();

    /**
     * Get the subdomains configuration. This only returns the subdomain
     * configuration created at startup from the local persisted data.
     * 
     * @return
     */
    public Set<SubDomainConfiguration> getSubDomainsConfiguration();

    /**
     * Get the sub domain configuration the local container belongs to.
     * 
     * @return
     */
    SubDomainConfiguration getSubDomainConfiguration();

    /**
     * Get a sub-domain configuration.
     * 
     * @param containerName
     *            the name of the sub-domain to get configuration from. If null, returns the local sub-domain
     *            configuration.
     * @return The sub-domain configuration of the sub-domain named <code>subdomainName</code>, or <code>null</code> if
     *         not found.
     */
    public SubDomainConfiguration getSubDomainConfiguration(final String subdomainName);

    /**
     * Get the domain configuration the local container belongs to.
     * 
     * @return
     */
    DomainConfiguration getDomainConfiguration();

    /**
     * @return The initial local topology read from the local topology definition (ie. topology.xml). Because of the
     *         dynamic topology loaded from the registry, the initial local topology can be not applied.
     */
    public Topology getInitialLocalTopology() throws ConfigurationException;

    /**
     * Set the running state of the local container.
     * 
     * @param state
     *            {@code ContainerConfiguration.ContainerState}
     */
    void setContainerState(ContainerConfiguration.ContainerState state);

    /**
     * Get the container configuration.
     * 
     * @param containerName
     *            the name of the container to get configuration from. If null, returns the local container
     *            configuration.
     * @return The container configuration of the container named <code>containerName</code>, or <code>null</code> if
     *         not found.
     */
    ContainerConfiguration getContainerConfiguration(final String containerName);

    /**
     * Registers a new container configuration within the local configuration.
     * 
     * @param addedContainer
     *            the container to register
     * 
     * @throws NoContainerConfigurationProvidedException
     *             No container configuration is provided (ie. <code>addedContainer</code> is <code>null</code>)
     * @throws InvalidContainerConfigurationProvidedException
     *             The container configuration provided (<code>addedContainer</code>) is invalid.
     * @throws ContainerAlreadyExistException
     *             The container to add already exists
     * @throws SubdomainUnknownException
     *             The sub-domain set in the container configuration is unknown, ie. the container is a member of a
     *             unknown sub-domain.
     */
    void addContainerConfiguration(final ContainerConfiguration addedContainer)
            throws NoContainerConfigurationProvidedException, InvalidContainerConfigurationProvidedException,
            ContainerAlreadyExistException, SubdomainUnknownException;

    /**
     * <p>
     * Unregisters the container within the local configuration.
     * </p>
     * <p>
     * The name of the container to remove <b>MUST</b> be defined in <code>removedContainer</code> otherwise the
     * exception {@link ContainerDoesNotExistException} will be thrown.
     * </p>
     * 
     * @param removedContainer
     *            The container to remove
     * 
     * @throws NoContainerConfigurationProvidedException
     *             No container configuration is provided(ie. <code>removedContainer</code> is <code>null</code>)
     * @throws ContainerDoesNotExistException
     *             The container to remove does not exist
     * @throws InvalidContainerConfigurationProvidedException
     *             The container configuration provided (<code>removedContainer</code>) is invalid.
     */
    void removeContainerConfiguration(final ContainerConfiguration removedContainer)
            throws NoContainerConfigurationProvidedException, ContainerDoesNotExistException,
            InvalidContainerConfigurationProvidedException;

    /**
     * Registers a new sub-domain within the local configuration.
     * 
     * @param addedSubdomain
     *            The sub-domain to register
     * 
     * @throws SubdomainAlreadyExistException
     *             The sub-domain to add already exists
     * @throws NoSubdomainConfigurationProvidedException
     *             No sub-domain configuration is provided
     * @throws InvalidSubdomainConfigurationProvidedException
     *             The sub-domain provided (<code>addedSubdomain</code>) is invalid.
     * @throws ConfigurationException
     *             Something is wrong
     */
    void addSubdomainConfiguration(final SubDomainConfiguration addedSubdomain) throws SubdomainAlreadyExistException,
            NoSubdomainConfigurationProvidedException, InvalidSubdomainConfigurationProvidedException;

    /**
     * <p>
     * Unregisters the sub-domain within the local configuration.
     * </p>
     * <p>
     * If the sub-domain contains containers, they are removed transitively.
     * </p>
     * <p>
     * The name of the sub-domain to remove <b>MUST</b> be defined in <code>removedSubdomain</code> otherwise the
     * exception {@link SubdomainDoesNotExistException} will be thrown.
     * </p>
     * 
     * @param removedSubdomain
     *            The sub-domain to remove
     * 
     * @throws NoSubdomainConfigurationProvidedException
     *             No sub-domain configuration is provided(ie. <code>removedSubdomain</code> is <code>null</code>)
     * @throws SubdomainDoesNotExistException
     *             The sub-domain to remove does not exist
     * @throws InvalidSubdomainConfigurationProvidedException
     *             The sub-domain configuration provided (<code>removedSubdomain</code>) is invalid.
     */
    public void removeSubdomainConfiguration(final SubDomainConfiguration removedSubdomain)
            throws NoSubdomainConfigurationProvidedException, SubdomainDoesNotExistException,
            InvalidSubdomainConfigurationProvidedException;

    /**
     * <p>
     * Create the container configuration based on the local topology and server configuration properties. Basic
     * consistency is controlled between the 2 configurations.
     * </p>
     * <p>
     * Required directories are created when loading the configuration: working area directory.
     * </p>
     * <p>
     * <b>Note 1</b>: If present into the server configuration properties, the parameter {@value #PROPERTY_TOPOLOGY_URL}
     * is ignored, the topology is provided through the parameter '<code>topology</code>'.
     * <p>
     * <b>Note 2: MUST be invoked just after having starting the Configuration service and before any other component
     * starts.</b>
     * </p>
     * <p>
     * <b>Note 2</b>: If a topology definition exists in the working area, the topology definition '
     * <code>topology</code>' is replaced by this persisted topology definition. The persisted topology aims to
     * persisted topology changes that can not be stored in the topology definition reference because we are not able to
     * manage all URL scheme.
     * </p>
     * 
     * @param serverLocalProperties
     *            The local properties of the current container (ie: the content of 'server.properties' or equivalent).
     *            Can be <code>null</code>.
     * @param topology
     *            The topology configuration. Can not be <code>null</code>.
     * @throws NoServerPropertiesProvidedException
     *             The server properties provided as argument are <code>null</code>.
     * @throws MissingServerPropertyException
     *             A server property is missing.
     * @throws NoTopologyProvidedException
     *             The topology provided as argument is <code>null</code>.
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    public void loadConfiguration(final Properties serverLocalProperties, final Topology topology)
            throws NoTopologyProvidedException, NoServerPropertiesProvidedException, MissingServerPropertyException,
            ConfigurationException;

    /**
     * <p>
     * Update the container configuration according to a topology definition. Basic consistency is controlled between
     * the existing topology and the new one.
     * </p>
     * 
     * @param topology
     *            The new topology to apply. Can not be <code>null</code>.
     * @throws NoTopologyProvidedException
     *             No topology is provided (ie. <code>topology</code> is null)
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    public void updateTopology(final Topology topology) throws NoTopologyProvidedException,
            InvalidSubdomainConfigurationProvidedException, ConfigurationException;

    /**
     * <p>
     * Reconfigure the current domain with the given one.
     * </p>
     * <p>
     * No checks is done before to reconfigure
     * </p>
     * 
     * @param newDomainConfiguration
     *            The new domain configuration to apply to the current domain
     * @throws NoDomainConfigurationProvidedException
     *             No domain configuration is provided
     * @throws NoDomainNameProvidedException
     *             No domain name is provided
     */
    public void reconfigureDomain(final DomainConfiguration newDomainConfiguration)
            throws NoDomainConfigurationProvidedException, NoDomainNameProvidedException;

    /**
     * <p>
     * Reconfigure the current sub-domain with the given one.
     * </p>
     * <p>
     * No checks is done before to reconfigure
     * </p>
     * 
     * @param newSubdomainConfiguration
     *            The new sub-domain configuration to apply to the current sub-domain
     * @throws NoSubdomainConfigurationProvidedException
     *             No sub-domain configuration is provided
     * @throws NoSubdomainNameProvidedException
     *             No sub-domain name is provided
     * @throws SubdomainAlreadyExistException
     *             The new sub-domain name already exists
     */
    public void reconfigureSubdomain(final SubDomainConfiguration newSubdomainConfiguration)
            throws NoSubdomainConfigurationProvidedException, NoSubdomainNameProvidedException,
            SubdomainAlreadyExistException;

    /**
     * <p>
     * Reconfigure the current container with the given sub-domain configuration.
     * </p>
     * <p>
     * No checks is done before to reconfigure
     * </p>
     * 
     * @param newSubdomainConfiguration
     *            The new sub-domain configuration to apply to the current container
     * @throws NoSubdomainConfigurationProvidedException
     *             No sub-domain configuration is provided
     * @throws NoSubdomainNameProvidedException
     *             No sub-domain name is provided
     * @throws SubdomainAlreadyExistException
     *             The new sub-domain name already exists
     */
    public void reconfigureContainer(final SubDomainConfiguration newSubdomainConfiguration)
            throws NoSubdomainConfigurationProvidedException, NoSubdomainNameProvidedException;
    
    /**
     * Validate the given security topology pass-phrase against the one configured at the current container.
     * 
     * @param securityPassPhrase
     *            The security topology pass-phrase to check.
     * @return <code>true</code> if the security topology pass-phrase is not null, non-empty and validated,
     *         <code>false</code> otherwise.
     */
    public boolean isSecurityTopologyPassphraseOk(final String securityPassPhrase);

    /**
     * Register a listener on events about remote container configuration of the sub-domain of the current container.
     * 
     * @param listener
     *            The listener to register
     */
    public void registerContainerConfigurationListener(final ContainerConfigurationListener listener);

    /**
     * Deregister a listener on events about remote container configuration of the sub-domain of the current container.
     * 
     * @param listener
     *            The listener to deregister
     */
    public void deregisterContainerConfigurationListener(final ContainerConfigurationListener listener);

    /**
     * <p>
     * Persists the given topology in the working area.
     * </p>
     * <p>
     * A such topology will be loaded before to check if a dynamic topology exists in the registry. See
     * {@link #loadConfiguration(Properties, Topology)}.
     * </p>
     * 
     * @param topology
     *            The topology to persist
     * @throws ConfigurationException
     *             An error occurs persisting the topology definition.
     */
    public void persistTopology(final Topology topology) throws ConfigurationException;

}
