/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container;

import java.util.Collection;
import java.util.Map;

import javax.management.ObjectName;

import org.objectweb.fractal.api.Component;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly;
import org.ow2.petals.microkernel.api.container.exception.ComponentLifecycleNotFoundException;
import org.ow2.petals.microkernel.api.container.exception.ContainerServiceException;
import org.ow2.petals.microkernel.api.container.exception.InstallerNotFoundException;
import org.ow2.petals.microkernel.api.container.exception.ServiceAssemblyLifecycleNotFoundException;
import org.ow2.petals.microkernel.api.container.exception.SharedLibraryLifecycleNotFoundException;

/**
 * This {@link ContainerService} interface allow to create and handle JBI containers.
 * @author Nicolas SALATGE - EBM WebSourcing
 */
public interface ContainerService {

    /**
     * Logger name of the container service
     */
    public static final String FRACTAL_COMPONENT_LOGGER_NAME = "Petals.Container.ContainerService";

    /**
     * Prefix of logger names of Fractal components associated to JBI artifacts
     */
    public static final String COMPONENT_LOGGER_PREFIX = "Petals.Container.Components";

    /**
     * Name of the server interface of the Fractal component associated to the container service
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Name of the client interface 'installers' of the Fractal component 'ContainerService', associated to the JBI
     * component installers.
     */
    public static final String INSTALLERS_ITF_PREFIX = "installers-";

    /**
     * Name of the client interface 'componentLifecycles' of the Fractal component 'ContainerService', associated to the
     * JBI component lifecycles.
     */
    public static final String COMPONENT_LIFECYCLES_ITF_PREFIX = "componentLifecycles-";

    /**
     * Name of the client interface 'serviceAssemblyLifecycles' of the Fractal component 'ContainerService', associated
     * to the JBI SA lifecycles.
     */
    public static final String SA_LIFECYCLES_ITF_PREFIX = "serviceAssemblyLifecycles-";

    /**
     * Name of the client interface 'sharedLibraryLifecycles' of the Fractal component 'ContainerService', associated to
     * the JBI SL lifecycles.
     */
    public static final String SL_LIFECYCLES_ITF_PREFIX = "sharedLibraryLifecycles-";

    /**
     * Create a JBI component installer and its associated Fractal component, from its component description.
     * 
     * @param componentDescription
     *            the component description of the JBI component
     * @return the installer of the JBI component
     * @throws ContainerServiceException
     *             An error occurs when creating the Fractal component associated to the installer of the JBI component
     */
    public Installer createInstaller(
            final org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription)
            throws ContainerServiceException;

    /**
     * Start the Fractal component associated to a JBI component installer
     * 
     * @param name
     *            The JBI component identifier
     * @throws InstallerNotFoundException
     *             The JBI component installer was not found
     * @throws ContainerServiceException
     *             An error occurs when starting the Fractal component associated to the installer of the JBI component
     * @return <code>true</code> if the Fractal component associated to the JBI component identifier was in state
     *         'stopped' and has been started, <code>false</code> if it was not in state 'started'.
     */
    public boolean startInstaller(final String name) throws InstallerNotFoundException, ContainerServiceException;

    /**
     * Register the MBean of a JBI component installer
     * 
     * @param name
     *            The JBI component identifier
     * @throws InstallerNotFoundException
     *             The JBI component installer was not found
     * @throws ContainerServiceException
     *             An error occurs when registering the MBean of the JBI component installer
     * @return The {@link ObjectName} of the MBean registered for the JBI component installer.
     */
    public ObjectName registerInstallerMBean(final String name) throws InstallerNotFoundException,
            ContainerServiceException;

    /**
     * Unregister the MBean of the a JBI component installer
     * 
     * @param name
     *            The JBI component identifier
     * @throws InstallerNotFoundException
     *             The JBI component installer was not found
     * @throws ContainerServiceException
     *             An error occurs when registering the MBean of the JBI component installer
     */
    public void unregisterInstallerMBean(final String name) throws InstallerNotFoundException,
            ContainerServiceException;

    /**
     * Remove an installer from the name of the component.
     * 
     * @param name
     *            the name of the component
     * @throws PetalsException
     *             : occurs when it is impossible to remove an installer
     */
    void removeInstaller(String name) throws PetalsException;

    /**
     * Create a JBI component life cycle and its associated Fractal component, from its component description.
     * 
     * @param componentDescription
     *            the component description of the JBI component
     * @return the component life cycle of the JBI component
     * @throws ContainerServiceException
     *             An error occurs when creating the Fractal component associated to the installer of the JBI component
     */
    ComponentLifeCycle createComponentLifeCycle(
            final org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription)
            throws ContainerServiceException;

    /**
     * Start the Fractal component associated to a JBI component life cycle
     * 
     * @param name
     *            The JBI component identifier
     * @throws ComponentLifecycleNotFoundException
     *             The JBI component life cycle was not found
     * @throws ContainerServiceException
     *             An error occurs when starting the Fractal component associated to the life cycle of the JBI component
     * @return <code>true</code> if the Fractal component associated to the JBI component identifier was in state
     *         'stopped' and has been started, <code>false</code> if it was not in state 'started'.
     */
    public boolean startComponentLifeCycle(final String name) throws ComponentLifecycleNotFoundException,
            ContainerServiceException;

    /**
     * Register the MBean of a JBI component life cycle
     * 
     * @param name
     *            The JBI component identifier
     * @throws ComponentLifecycleNotFoundException
     *             The JBI component life cycle was not found
     * @throws ContainerServiceException
     *             An error occurs when registering the MBean of the JBI component life cycle
     * @return The {@link ObjectName} of the MBean registered for the JBI component life cycle.
     */
    public ObjectName registerComponentLifeCycleMBean(final String name) throws ComponentLifecycleNotFoundException,
            ContainerServiceException;

    /**
     * Unregister the MBean of a JBI component life cycle
     * 
     * @param name
     *            The JBI component identifier
     * @throws ComponentLifecycleNotFoundException
     *             The JBI component life cycle was not found
     * @throws ContainerServiceException
     *             An error occurs when registering the MBean of the JBI component life cycle
     */
    public void unregisterComponentLifeCycleMBean(final String name) throws ComponentLifecycleNotFoundException,
            ContainerServiceException;

    /**
     * Remove a JBI component life cycle.
     * 
     * @param name
     *            the name of the component
     * @throws PetalsException
     *             : occurs when it is impossible to remove a component life cycle
     */
    void removeComponentLifeCycle(final String name) throws PetalsException;

    /**
     * Get the container component.
     * @return the parent component (the fractal container)
     */
    Component getParentContainer();

    /**
     * Create a JBI service assembly life cycle and its associated Fractal component, from its description.
     * 
     * @param sa
     *            the description of the JBI service assembly
     * @return the life cycle of the JBI service assembly
     * @throws ContainerServiceException
     *             An error occurs when creating the Fractal component associated to the life cycle of the JBI service
     *             assembly
     */
    public ServiceAssemblyLifeCycle createServiceAssemblyLifeCycle(final ServiceAssembly sa)
            throws ContainerServiceException;

    /**
     * Start the Fractal component associated to a JBI service assembly life cycle
     * 
     * @param name
     *            The JBI service assembly identifier
     * @throws ServiceAssemblyLifecycleNotFoundException
     *             The JBI service assembly life cycle was not found
     * @throws ContainerServiceException
     *             An error occurs when starting the Fractal component associated to the life cycle of the JBI service
     *             assembly
     * @return <code>true</code> if the Fractal component associated to the JBI service assembly identifier was in state
     *         'stopped' and has been started, <code>false</code> if it was not in state 'started'.
     */
    public boolean startServiceAssemblyLifeCycle(final String name) throws ServiceAssemblyLifecycleNotFoundException,
            ContainerServiceException;

    /**
     * remove a service assembly life cycle Fractal component.
     * 
     * @param sa
     *            the service assembly name
     * @throws PetalsException
     *             occurs when it is impossible to remove a service assembly life cycle Fractal component
     */
    void removeServiceAssemblyLifeCycle(final String saName) throws PetalsException;

    /**
     * Create a JBI shared library life cycle and its associated Fractal component, from its description.
     * 
     * @param sl
     *            the description of the JBI shared library
     * @return the life cycle of the JBI shared library
     * @throws ContainerServiceException
     *             An error occurs when creating the Fractal component associated to the life cycle of the JBI shared
     *             library
     */
    public SharedLibraryLifeCycle createSharedLibraryLifeCycle(final SharedLibrary sl) throws ContainerServiceException;

    /**
     * Start the Fractal component associated to a JBI shared library life cycle
     * 
     * @param slName
     *            The JBI shared library identifier
     * @param slVersion
     *            The version of the shared library
     * @throws SharedLibraryLifecycleNotFoundException
     *             The JBI shared library life cycle was not found
     * @throws ContainerServiceException
     *             An error occurs when starting the Fractal component associated to the life cycle of the JBI shared
     *             library
     * @return <code>true</code> if the Fractal component associated to the JBI component identifier was in state
     *         'stopped' and has been started, <code>false</code> if it was not in state 'started'.
     */
    public boolean startSharedLibraryLifeCycle(final String slName, final String slVersion)
            throws SharedLibraryLifecycleNotFoundException, ContainerServiceException;

    /**
     * Remove a shared library life cycle
     * 
     * @param slName
     *            the shared library name
     * @param slVersion
     *            the shared library version
     * @throws PetalsException
     *             occurs when it is impossible to remove a shared library life cycle
     */
    public void removeSharedLibraryLifeCycle(final String slName, final String slVersion) throws PetalsException;

    /**
     * Get a JBI component installer ({@link Installer}) by its component name.
     * 
     * @param name
     *            The JBI component installer to find
     * @return the {@link Installer}, or <code>null</code> if not found
     */
    public Installer getInstallerByName(final String name);

    /**
     * Get the {@link Installer}s, the map key is the JBI component name identifier
     * 
     * @return
     */
    public Map<String, Installer> getInstallers();

    /**
     * Get the list of {@link ComponentLifeCycle}s
     * 
     * @return The list of {@link ComponentLifeCycle}s, not <code>null</code>.
     */
    public Map<String, ComponentLifeCycle> getComponentLifecycles();

    /**
     * Get a JBI component life cycle ({@link ComponentLifeCycle}) by its component name.
     * 
     * @param name
     *            The JBI component life cycle to find
     * @return the {@link ComponentLifeCycle}, or <code>null</code> if not found
     */
    public ComponentLifeCycle getComponentLifecycleByName(final String name);

    /**
     * Get the list of service engine {@link ComponentLifeCycle}s
     * 
     * @return The list of service engine {@link ComponentLifeCycle}s, not <code>null</code>.
     */
    public Collection<String> getServiceEnginesNames();

    /**
     * Get the list of binding component {@link ComponentLifeCycle}s
     * 
     * @return The list of binding component {@link ComponentLifeCycle}s, not <code>null</code>.
     */
    public Collection<String> getBindingComponentsNames();

    /**
     * Check if a JBI component is a service engine
     * 
     * @param componentName
     *            The JBI component identifier, not <code>null</code>
     * @return <code>true</code> if the component is a service engine, <code>false</code> otherwise.
     */
    public boolean isServiceEngine(final String componentName);

    /**
     * Check if a JBI component is a binding component
     * 
     * @param componentName
     *            The JBI component identifier, not <code>null</code>
     * @return <code>true</code> if the component is a binding component, <code>false</code> otherwise.
     */
    public boolean isBindingComponent(final String componentName);

    /**
     * Get the JBI service assembly life cycle ({@link ServiceAssemblyLifeCycle}) by its unique name.
     * 
     * @param name
     *            The JBI service assembly identifier
     * @return The {@link ServiceAssemblyLifeCycle}, or <code>null</code> if not found
     */
    public ServiceAssemblyLifeCycle getServiceAssemblyByName(final String saName);

    /**
     * Get the map of {@link ServiceAssemblyLifeCycle}s by name
     * 
     * @return The list of {@link ServiceAssemblyLifeCycle}s, not <code>null</code>.
     */
    public Map<String, ServiceAssemblyLifeCycle> getServiceAssemblies();

    /**
     * Get the {@link SharedLibraryLifeCycle} by its unique name.
     * 
     * @param slName
     *            The JBI shared library identifier
     * @param slVersionName
     *            The version of the JBI shared library
     * @return the {@link SharedLibraryLifeCycle}, or <code>null</code> if not found
     */
    public SharedLibraryLifeCycle getSharedLibraryByName(final String slName, final String slVersion);

    /**
     * Get the map of {@link SharedLibraryLifeCycle}s by names.
     * 
     * @return the map, not <code>null</code>
     */
    public Map<SharedLibraryBean, SharedLibraryLifeCycle> getSharedLibraries();

}
