/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container;

import java.io.File;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.jbi.management.LifeCycleMBean;

import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;

/**
 * This class manage the life cycle of deployed service units.
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class ServiceUnitLifeCycle implements LifeCycleMBean {

    private ContainerService containerService;

    private ServiceUnit serviceUnit;

    private String serviceUnitRootPath;

    private Jbi jbiDescriptor;

    private ServiceUnitManager serviceUnitManager;

    private String name;

    private String state;

    public ServiceUnitLifeCycle(ServiceUnit serviceUnit, String serviceUnitRootPath,
            final ContainerService containerService) {
        super();
        this.containerService = containerService;
        this.serviceUnit = serviceUnit;
        this.serviceUnitRootPath = serviceUnitRootPath;
        if (serviceUnit != null && serviceUnit.getIdentification() != null) {
            this.name = serviceUnit.getIdentification().getName();
        }
        state = SHUTDOWN;
    }

    /**
     * Returns the ServiceUnitManager associated with the service unit's target
     * component
     * 
     * @return null if there's no Service Unit Manager
     */
    public ServiceUnitManager getServiceUnitManager() {
        if (this.serviceUnitManager == null) {
            Installer installer = containerService.getInstallerByName(serviceUnit.getTarget()
                    .getComponentName());

            if (installer != null) {
                this.serviceUnitManager = installer.getComponent().getServiceUnitManager();
            }
        }

        return this.serviceUnitManager;
    }

    /**
     * Return the name of the targeted component.
     * 
     * @return the name of the targeted component
     */
    public String getTargetComponentName() {
        return serviceUnit.getTarget().getComponentName();
    }

    public String getServiceUnitRootPath() {
        return serviceUnitRootPath;
    }

    public String getSuName() {
        return name;
    }

    public synchronized Jbi getServiceUnitDescriptor() {
        if (this.jbiDescriptor == null) {
            try {
                this.jbiDescriptor = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromFolder(
                        new File(this.serviceUnitRootPath));
            } catch (Exception e) {
                // not possible, the descriptor has been already loaded by the
                // target component
            }
        }
        return jbiDescriptor;
    }

    public void init() throws DeploymentException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().init(name, serviceUnitRootPath);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = STOPPED;
    }

    public String getCurrentState() {
        return state;
    }

    public void shutDown() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().shutDown(name);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = SHUTDOWN;
    }

    public void start() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().start(name);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = STARTED;
    }

    public void stop() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().stop(name);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = STOPPED;
    }

    /**
     * @return the state
     */
    public String getState() {
        return this.state;
    }

}
