/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.thread;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;

import javax.jbi.JBIException;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * This class is the parent class for managing tasks to perform on Components,
 * Bootstrap and ServiceAssemblies. Tasks on those objects are performed in a
 * dedicated thread, preventing the main thread to be blocked. When a timeout
 * exceeds on an action, the task is considered as blocked, and an exception is
 * thrown
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public abstract class AbstractThread extends AbstractLoggedThread {

    /**
     * Exception thrown when a task failed in its own thread. Retrieved by the
     * main thread
     */
    protected JBIException jbiException;

    /**
     * The blocking working queue is equal to a FIFO. The worker thread is
     * waiting for an element to be added in this queue by the caller thread.
     * 
     * This queue is the synchronization object between caller thread and
     * execution thread. When an element is added by the caller thread, the
     * execution thread wakes up and do the task.
     */
    protected final BlockingQueue<String> requestQueue;

    /**
     * The response queue used to wait for the task to complete. The caller
     * thread is blocked on this queue while the task is not complted by the
     * worker thread.
     * 
     * This queue is the synchronization object between execution thread and
     * caller thread. When the execution thread put an object in this queue, the
     * caller thread wakes up.
     */
    protected final BlockingQueue<String> responseQueue;

    /**
     * Shutdown thread action
     */
    public static final String SHUTDOWNTHREAD = "SHUTDOWNTHREAD";

    /**
     * Time before an action is considered as blocked Exception is thrown when
     * this time exceed
     */
    protected final long timeout;

    /**
     * Creates a new instance of AbstractThread
     */
    public AbstractThread(final long taskTimeout, final LoggingUtil logger) {
        super(logger);
        this.requestQueue = new ArrayBlockingQueue<String>(1);
        this.responseQueue = new ArrayBlockingQueue<String>(1);
        this.timeout = taskTimeout;
    }

    @Override
    public void run() {
        boolean run = true;
        while (run) {
            try {
                // waits until there is one element available in the request
                // queue
                final String action = this.requestQueue.take();
                // if return false, stop the loop and so stop the current
                // thread
                run = doTask(action);

                // put element in response element wakes up the caller thread
                this.responseQueue.put(action);
            } catch (InterruptedException e) {
                getLogger().error(
                        "The life-cycle thread '" + this.getName()
                        + "' has been interrupted, probably due to a reached timeout");
            } catch (Exception e) {
                getLogger().error(e);
            }
        }
        return;
    }

    /**
     * Execute the task in the dedicated thread. A JBI exception can occur
     * during the task execution. In this case, the local jbiException must be
     * filled with this exception and is shared between client and thread.
     * 
     * @return false if the current thread needs to be shutdowned
     */
    protected abstract boolean doTask(String action);

    /**
     * Execute the action. If the thread is not alive, throw an exception. On
     * the other cases, put the action in the FIFO and wait for the task to
     * complete. This method is the entry point of the task submission, multiple
     * threads will wait for other threads to complete.
     * 
     * @param action
     */
    protected void execute(final String action) throws JBIException {
        /* thread is alive test */
        if (!this.isAlive()) {
            throw new JBIException("The thread '" + this.getName() + "' is dead");
        }

        try {
            /*
             * put action in request FIFO, this will wait for an available place
             * in the queue. Since the queue is one element size, block if there
             * is a current task execution.
             */
            this.requestQueue.put(action);

            /*
             * wait response == wait end of execution of the current query,
             * waiting for the doTask method to complete to be sure that tasks
             * are not done simultaneously. if timeout exceed (the underlying
             * operation is considered as blocked), throw an exception and kill
             * the thread.
             */
            final String actionPerformed;
            if (this.timeout > 0L) {
                actionPerformed = this.responseQueue.poll(this.timeout, TimeUnit.MILLISECONDS);
            } else {
                actionPerformed = this.responseQueue.take();
            }

            if (actionPerformed == null) {
                this.interrupt();
                throw new JBIException("Timeout exceeded when executing " + action
                        + " management task. Assuming the task is blocked.");
            }

            /*
             * Check if the underlying task executed in the dedicated thread
             * thrown an exception; throws it if it exists.
             */
            if (this.jbiException != null) {
                final JBIException e = this.jbiException;
                this.jbiException = null;
                throw e;
            }
        } catch (final InterruptedException e1) {
            throw new JBIException("Interrupted exception during " + action + " task execution.",
                    e1);
        }
    }

    /**
     * Shutdown the current thread
     * 
     */
    public void shutdownThread() throws JBIException {
        this.execute(SHUTDOWNTHREAD);
    }
}
