/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.messaging;

import java.util.List;

import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.jbi.component.PetalsComponentContext;

/**
 * Routes messages from the <code>DeliveryChannel</code> to their destinations.
 * <p>
 * Perform an address resolution, finding the endpoint(s) satisfying the
 * destination information, and select the appropriate transporter to convey the
 * messages. Resolutions are based on defined rules.
 * </p>
 * <p>
 * Routes messages pushed from the <code>Transporter</code> to the target
 * <code>DeliveryChannel</code>.
 * </p>
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public interface RouterService extends org.ow2.petals.clientserverapi.jbi.messaging.RouterService {
    
    public static final String COMPONENT_LOGGER_NAME = "Petals.JBI-Messaging.RouterService";

    /**
     * The message exchange property to set the TTL of the exchange.
     */
    static final String PROPERTY_ROUTER_TIMEOUT_DEADLINE = "org.ow2.petals.router.timeout-deadline";

    /**
     * The message exchange property name to set the provider link type to
     * resolve. See JBI specification section 5.4.3.3 for more details.
     */
    static final String PROPERTY_ROUTER_PROVIDER_LINKTYPE = "org.ow2.petals.router.provider.linktype";

    /**
     * Constant used to specify the property name to set the router policy. Used
     * only in plaform distribution. The default policy is fast.
     */
    static final String PROPERTY_ROUTER_QOS = "org.ow2.petals.router.qos";

    /**
     * Constant used to specify the property name to set the router send
     * attempt. Used only in plaform distribution. The default attempt is 2.
     */
    static final String PROPERTY_ROUTER_SEND_ATTEMPT = "org.ow2.petals.router.send.attempt";

    /**
     * Constant used to specify the property name to set the delay between
     * router send attempts. Used only in plaform distribution. The default
     * attempt is 1 second.
     */
    static final String PROPERTY_ROUTER_SEND_DELAY = "org.ow2.petals.router.send.delay";

    /**
     * Send the message into the NMR.
     * 
     * @param componentContext
     *            the source component context
     * @param exchange
     * @throws RoutingException
     *             impossible to send the message
     */
    void send(PetalsComponentContext componentContext, PetalsMessageExchange exchange)
            throws RoutingException;

    /**
     * Send the message synchronously into the NMR.
     * 
     * @param componentContext
     *            the source component context
     * @param exchange
     * @param timeOut
     *            timeout of the message (in ms). 0 for no timeout
     * @return the new message exchange (which can be the same as the one sent) or <code>null</code> if there was a
     *         timeout
     * 
     * @throws RoutingException
     *             impossible to process a synchronous send
     */
    PetalsMessageExchange sendSync(PetalsComponentContext componentContext, PetalsMessageExchange exchange,
            long timeout) throws RoutingException, InterruptedException;

    /**
     * Ask to receive a message from MNR and return it.
     * 
     * @param componentContext
     *            The source component context <b>MUST</b> be non null
     * @param timeoutMS
     *            parameter which represents
     *            <ul>
     *            <li>The time to wait if greater than 0</li>
     *            <li>A non blocking call if equal to 0</li>
     *            <li>Infinite Blocking call if less than 0</li>
     *            </ul>
     * 
     * @return The message exchange retrieved from the <code>Transporter</code> or null if it timed out.
     * 
     * @throws RoutingException
     *             impossible to receive the message
     */
    PetalsMessageExchange receive(PetalsComponentContext componentContext, long timeoutMS)
            throws RoutingException, InterruptedException;

    /**
     * Inform that a component is installing. The <code>Router</code> must
     * inform its <code>Transporters</code> about the component installation.
     * 
     * @param componentContext
     *            The component context
     * 
     * @throws RoutingException
     *             impossible to add the component
     */
    void addComponent(PetalsComponentContext componentContext) throws RoutingException;

    /**
     * Inform that a component is uninstalling. The <code>Router</code> must
     * inform its <code>Transporters</code> about the component uninstallation.
     * 
     * @param componentContext
     *            The component context
     * 
     * @throws RoutingException
     *             impossible to remove the component
     */
    void removeComponent(PetalsComponentContext componentContext) throws RoutingException;

    /**
     * Inform that a SA life-cycle has changed its state.
     * 
     * @param serviceUnitDescriptors
     *            the list of bound SU JBI descriptors
     */
    void modifiedSALifeCycle(List<ServiceUnitLifeCycle> serviceUnitLifes);

    /**
     * Prepare the stopping of the component. It means that all ongoing message
     * traffic must terminate during the method invocation and all new message
     * traffic is rejected.
     */
    void stopTraffic();
}
