/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.messaging.registry;

import java.util.List;

import javax.jbi.JBIException;
import javax.xml.namespace.QName;

import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.implementation.MutableImplementation;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.exception.DynamicTopologyNotLockedByCurrentThreadException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.exception.DynamicTopologyNotLockedException;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.topology.generated.Topology;
import org.w3c.dom.Document;

/**
 * This class is used to manage everything that deals with endpoint in the distributed environment.
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 */
public interface EndpointRegistry extends EndpointRegistryMBean, MutableImplementation {

    public static final String COMPONENT_LOGGER_NAME = "Petals.System.Registry";

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#REGISTRY_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * The name of the client interface of the component {@value FractalHelper#REGISTRY_COMPONENT} for the configuration
     * service.
     */
    public final static String CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Get the list of registry listeners. This can be used iin the
     * implementation to do special things such as monitoring etc...
     * 
     * @return
     */
    List<RegistryListener> getListeners();

    List<PetalsServiceEndpoint> getInternalEndpoints() throws RegistryException;

    List<PetalsServiceEndpoint> getExternalEndpoints() throws RegistryException;

    List<PetalsServiceEndpoint> getEndpoints() throws RegistryException;
    
    /**
     * <p>
     * Register an endpoint in the endpoints registry.
     * </p>
     * <p>
     * <b>Note</b>: If the endpoint already exists and <code>canOverwrite</code> is set to <code>true</code>, it is
     * overridden. Otherwise the error {@link EndpointAlreadyExistsException} will occur. The overwriting is required by
     * the recovering service.
     * 
     * @param serviceName
     *            Endpoint service name.
     * @param endpointName
     *            Endpoint name
     * @param interfaces
     *            Endpoint interfaces
     * @param description
     *            Endpoint WSDL
     * @param address
     * @param canOverwrite
     *            If set to <code>true</code>, if the endpoint already exists, it will be overwritten, otherwise the
     *            error {@link EndpointAlreadyExistsException} is thrown.
     * @return A {@link PetalsServiceEndpoint} associated to the registered endpoint.
     * @throws EndpointAlreadyExistsException
     *             The endpoint already exists, and <code>canOverwrite</code> is not set to <code>true</code>.
     * @throws RegistryException
     */
    public PetalsServiceEndpoint activateEndpoint(final QName serviceName, final String endpointName,
            final List<QName> interfaces, final Document description, final PetalsServiceEndpoint address,
            final boolean canOverwrite) throws EndpointAlreadyExistsException, RegistryException;

    /**
     * Deactivates the local given endpoint. Does not care about remotes ones !
     * 
     * 
     * @throws EndpointDoesNotExistsException
     *             The endpoint is not already registered.
     * @throws JBIException
     *             if the endpoint cannot be deactivated
     */
    void deactivateEndpoint(String endpointName, QName serviceName) throws EndpointDoesNotExistsException,
            RegistryException;

    /**
     * Register this given external endpoint
     * 
     * @param externalEndpoint
     *            the external endpoint to be registered; must be non null
     * @throws JBIException
     *             if an external endpoint wit the same name is already
     *             registered
     */
    void registerExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint serviceEndpoint)
            throws RegistryException;

    /**
     * Deregister external endpoint
     * 
     * @param endpoint
     *            external to be deregistered; must be non null
     * @throws JBIException
     *             if the endpoint cannot be deregistered
     */
    void deregisterExternalEndpoint(String endpointName, QName serviceName)
            throws RegistryException;

    /**
     * Register a connection
     * 
     * @param consInterface
     *            consumer interface name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if a problem occurs in the registry
     */
    void registerConnection(QName consInterface, QName provService, String provEndpoint)
            throws RegistryException;

    /**
     * Register a connection
     * 
     * @param consService
     *            consumer service name; must be non null
     * @param consEndpoint
     *            consumer endpoint name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if a problem occurs in the registry
     */
    void registerConnection(QName consService, String consEndpoint, QName provService,
            String provEndpoint) throws RegistryException;

    /**
     * Delete a connection
     * 
     * @param consInterface
     *            consumer interface name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if the connection can not be deleted
     */
    void deregisterConnection(QName consInterface, QName provService, String provEndpoint)
            throws RegistryException;

    /**
     * Delete a connection
     * 
     * @param consService
     *            consumer service name; must be non null
     * @param consEndpoint
     *            consumer endpoint name; must be non null
     * 
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws JBIException
     *             if the connection can not be deleted
     */
    void deregisterConnection(QName consService, String consEndpoint, QName provService,
            String provEndpoint) throws RegistryException;

    /**
     * Return the descriptor for a specified endpoint. It can not be a
     * connection endpoint.
     * 
     * @param endpoint
     *            endpoint on which we want to retrieve the descriptor; must be
     *            non null
     * @return Document the DOM representation of the descriptor (WSDL)
     */
    Document getEndpointDescriptorForEndpoint(PetalsServiceEndpoint endpoint) throws RegistryException;

    /**
     * Return the interfaces for the specified endpoint. If no interface is set in the provided service endpoint, this
     * will get this information from the registry. The endpoint information is not updated with the new data !
     * 
     * @param endpoint
     *            endpoint on which we want to retrieve the interface names; must be non null and not a connection
     *            endpoint
     * @return Array of interfaces QName, or <code>null</code> is the service endpoint can not be found in the registry
     */
    QName[] getInterfacesForEndpoint(PetalsServiceEndpoint svcEp);

    /**
     * Queries the registry for active endpoints that implements the given interface, according to the link type.
     * 
     * @param interfaceName
     *            qualified interface/portType that is implemented by the endpoints; if null then all internal activated
     *            endpoints in the JBI environment must be returned
     * @return an array of available endpoints for the specified interface name; must be non null, may be empty
     */
    PetalsServiceEndpoint[] getInternalEndpointsForInterface(QName interfaceName, LinkType linktype)
            throws RegistryException;

    /**
     * Queries the registry for active endpoints belonging to the given service,
     * according to the link type.
     * 
     * @param serviceName
     *            qualified name of the service that the endpoints are part of;
     *            must be non null
     * @return an array of available endpoints for the specified service name;
     *         must be non null, may be empty
     */
    PetalsServiceEndpoint[] getInternalEndpointsForService(QName serviceName, LinkType linktype)
            throws RegistryException;

    /**
     * Queries the registry for external endpoints that implements the given interface name
     * 
     * @param interfaceName
     *            qualified name of interface implemented by the endpoints; can be null
     * @return an array of available endpoints for the specified interface name; returns all external available
     *         endpoints if interface name is null; must be non null, may be empty
     */
    PetalsServiceEndpoint[] getExternalEndpointsForInterface(QName interfaceName)
            throws RegistryException;

    /**
     * Queries the registry for external endpoints that are part of the given
     * service name
     * 
     * @param serviceName
     *            qualified name of service that contains endpoints; must be non
     *            null
     * @return an array of available endpoints for the specified service name;
     *         must be non null, may be empy
     */
    PetalsServiceEndpoint[] getExternalEndpointsForService(QName serviceName) throws RegistryException;

    /**
     * Get the service endpoint for the named activated endpoint, if any. It can
     * be a connection.
     * 
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @return the named endpoint, or null if the named endpoint is not
     *         activated
     */
    PetalsServiceEndpoint getEndpoint(QName service, String name) throws RegistryException;

    /**
     * Remove all the local endpoints. If the registry is a distributed one,
     * propagate the delete action to them.
     * 
     * @return
     * @throws RegistryException
     */
    void removeAllLocalEndpoints() throws RegistryException;

    /**
     * <p>
     * Lock the dynamic topology objects.
     * </p>
     * <p>
     * This method <b>MUST</b> be invoked before to update the dynamic topology, otherwise the exception
     * {@link DynamicTopologyNotLockedException} is thrown.
     * </p>
     * 
     * @return <code>true</code> if a lock has been acquired, <code>false</code> otherwise
     * @throws InterruptedException
     *             The current thread has been interrupted
     */
    public boolean lockDynamicTopology() throws InterruptedException;

    /**
     * Unlock the dynamic topology objects
     * 
     * @throws DynamicTopologyNotLockedException
     *             The dynamic topology objects are not locked.
     * @throws DynamicTopologyNotLockedByCurrentThreadException
     *             The dynamic topology objects was not locked by the thread that try to unlock them.
     */
    public void unlockDynamicTopology() throws DynamicTopologyNotLockedException,
            DynamicTopologyNotLockedByCurrentThreadException;

    /**
     * Set the dynamic topology. If a dynamic topology already exists, it is overridden, and all clients of the registry
     * are informed of the change (old value vs new value).
     * 
     * @param topology
     *            The topology to use as dynamic topology
     * @throws DynamicTopologyNotLockedException
     *             The dynamic topology objects are not locked.
     * @throws DynamicTopologyNotLockedByCurrentThreadException
     *             The dynamic topology objects was not locked by the thread that try to unlock them.
     */
    public void setDynamicTopology(final Topology topology) throws DynamicTopologyNotLockedException,
            DynamicTopologyNotLockedByCurrentThreadException;

    /**
     * @return The current dynamic topology. Can be <code>null</code>
     */
    public Topology getDynamicTopology();

}
