/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.microkernel.api.jbi.servicedesc;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * The JBI implementation of the petals service endpoint.
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class JBIServiceEndpointImpl implements PetalsServiceEndpoint, Serializable {

    private static final String JBI_NAMESPACE = "http://java.sun.com/xml/ns/jbi";
    private static final String EPR_NAME = "endpoint-reference";
    private static final String SERVICE_NAME = "service-name";
    private static final String ENDPOINT_NAME = "endpoint-name";

    /**
     * Unique serial UID
     */
    private static final long serialVersionUID = 4487649085296224233L;

    /**
     * This attribute is defined "transient" to avoid its serialization when sending message exchanges. This attribute
     * is saved/loaded from the registry by the endpoint service.
     */
    private transient Document description;

    private String endpointName;

    private QName serviceName;

    private Location location;

    private List<QName> interfacesName;

    private EndpointType type;

    public JBIServiceEndpointImpl() {
        this(null, null, new ArrayList<QName>(0), null, new Location());
    }

    public JBIServiceEndpointImpl(String endpointName, QName serviceName, List<QName> interfacesName,
            Document description, Location location) {
        this.endpointName = endpointName;
        this.serviceName = serviceName;
        this.interfacesName = interfacesName;
        this.description = description;
        this.location = location;
        this.type = EndpointType.INTERNAL;
    }

    @Override
    public Document getDescription() {
        return this.description;
    }

    @Override
    public String getEndpointName() {
        return this.endpointName;
    }

    @Override
    public Location getLocation() {
        return this.location;
    }

    @Override
    public QName getServiceName() {
        return this.serviceName;
    }

    @Override
    public QName[] getInterfaces() {
        return this.interfacesName.toArray(new QName[this.interfacesName.size()]);
    }

    @Override
    public String toString() {
        return this.serviceName + " ->" + this.endpointName + " (" + this.getType() + "):"
                + this.location.getSubdomainName() + "/" + this.location.getContainerName() + "/"
                + this.location.getComponentName();
    }

    @Override
    public EndpointType getType() {
        return this.type;
    }

    @Override
    public void setType(EndpointType type) {
        this.type = type;
    }

    @Override
    public DocumentFragment getAsReference(final QName operationName) {
        Document doc = DocumentBuilders.newDocument();
        if (doc == null) {
            return null;
        }
        DocumentFragment result = doc.createDocumentFragment();
        Element e = doc.createElementNS(JBI_NAMESPACE, EPR_NAME);
        e.setAttribute(SERVICE_NAME, this.getServiceName().getLocalPart());
        e.setAttribute(ENDPOINT_NAME, this.getEndpointName());
        result.appendChild(e);
        return result;
    }

    public void setDescription(Document description) {
        this.description = description;
    }

    public void setEndpointName(String endpointName) {
        this.endpointName = endpointName;
    }

    public void setServiceName(QName serviceName) {
        this.serviceName = serviceName;
    }

    public void setLocation(Location location) {
        this.location = location;
    }

    public void setInterfacesName(QName[] interfacesName) {
        // caution, asList does not copy the content of the array
        this.setInterfacesName(Arrays.asList(interfacesName));
    }

    public void setInterfacesName(List<QName> interfacesName) {
        // let's make a copy
        this.interfacesName = new ArrayList<>(interfacesName);
    }
}
