/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.system;

import java.net.URL;
import java.util.List;

import org.ow2.petals.systemstate.SystemStateException;
import org.ow2.petals.systemstate.generated.Component;
import org.ow2.petals.systemstate.generated.ServiceAssembly;
import org.ow2.petals.systemstate.generated.SharedLibrary;

/**
 * Allows to persist, update, delete and recover JBI artefacts.
 * @author Olivier Fabre - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public interface SystemStateService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.System.SystemStateService";

    /**
     * Name of the server interface of the Fractal component associated to the system state service
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Create a {@link Component} entry in the System State.
     * 
     * @param componentName
     * @param installUrl
     * @param archiveName
     * 
     * @return the created {@link Component} entry
     * @throws SystemStateException
     *             failed to persist the new System State
     */
    Component createComponentStateHolder(String componentName, URL installUrl, String archiveName)
            throws SystemStateException;

    /**
     * Create a {@link ServiceAssembly} entry in the System State.
     * 
     * @param saName
     * @param installUrl
     * @param archiveName
     * 
     * @return the created {@link ServiceAssembly}
     * @throws SystemStateException
     *             failed to persist the new System State
     */
    ServiceAssembly createServiceAssemblyStateHolder(String saName, URL installUrl, String archiveName)
            throws SystemStateException;

    /**
     * Create a {@link SharedLibrary} entry in the System State.
     * 
     * @param slName
     * @param installUrl
     * @param archiveName
     * 
     * @return the created {@link SharedLibrary}
     * @throws SystemStateException
     *             failed to persist the new System State
     */
    SharedLibrary createSharedLibraryStateHolder(String slName, String version, URL installUrl, String archiveName)
            throws SystemStateException;

    /**
     * Delete a {@link Component} entry from the System State.
     * 
     * @param componentName
     * @return the deleted entry or {@code null} if not found
     * @throws SystemStateException
     *             failed to persist the new System State
     */
    Component deleteComponentStateHolder(String componentName) throws SystemStateException;

    /**
     * Delete a {@link ServiceAssembly} entry from the System State.
     * 
     * @param saName
     * @return the deleted entry or {@code null} if not found
     * @throws SystemStateException
     *             failed to persist the new System State
     */
    ServiceAssembly deleteServiceAssemblyStateHolder(String saName) throws SystemStateException;

    /**
     * Delete a {@link SharedLibrary} entry from the System State.
     * 
     * @param slName
     * @param version
     * @return the deleted entry or {@code null} if not found
     * @throws SystemStateException
     *             failed to persist the new System State
     */
    SharedLibrary deleteSharedLibraryStateHolder(String slName, String version)
            throws SystemStateException;

    /**
     * Get the {@link Component} entry from the System State.
     * 
     * @param componentName
     * @return the {@link Component} entry or {@code null} if not found
     */
    Component getComponentStateHolder(String componentName);

    /**
     * Get the {@link Component}s entries from the System State.
     * 
     * @return the list of {@link Component}s entries
     */
    List<Component> getComponentStateHolders();

    /**
     * Get the {@link ServiceAssembly} entry from the System State.
     * 
     * @param saName
     * @return the {@link ServiceAssembly} entry or {@code null} if not found
     */
    ServiceAssembly getServiceAssemblyStateHolder(String saName);

    /**
     * Get the {@link ServiceAssembly}s entries from the System State.
     * 
     * @return the list of {@link ServiceAssembly}s entries
     */
    List<ServiceAssembly> getServiceAssemblyStateHolders();

    /**
     * Get the {@link SharedLibrary} entry from the System State.
     * 
     * @param slName
     * @param version
     * @return the {@link SharedLibrary} entry or {@code null} if not found
     */
    SharedLibrary getSharedLibraryStateHolder(String slName, String version);

    /**
     * Get the {@link SharedLibrary}s entries from the System State.
     * 
     * @return the list of {@link SharedLibrary}s entries
     */
    List<SharedLibrary> getSharedLibraryStateHolders();

    /**
     * Get the {@link SharedLibrary}s entries from the System State matching the
     * given name.
     * 
     * @param slName
     * @return the list of {@link SharedLibrary}s entries 
     */
    List<SharedLibrary> getSharedLibraryStateHolders(String slName);

    /**
     * Check if the given component is already installed
     * 
     * @param componentName
     * @return {@code true} if already installed, {@code false} otherwise
     */
    boolean isComponentInstalled(String componentName);

    /**
     * Update the given component installation state.
     * 
     * @param componentName
     * @param installState
     */
    void updateComponentInstallationState(String componentName, String installState)
            throws SystemStateException;

    /**
     * Update the given component lifecycle state.
     * 
     * @param componentName
     * @param lifecycleState
     */
    void updateComponentLifeCycleState(String componentName, String lifecycleState)
            throws SystemStateException;

    /**
     * Update the given service assembly state.
     * 
     * @param saName
     * @param lifecycleState
     */
    void updateServiceAssemblyState(String saName, String lifecycleState)
            throws SystemStateException;

}
