/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.system.persistence;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;

/**
 * The Persistent Service.
 * @author Roland Naudin - EBM WebSourcing
 * @since 3.1
 */
public interface PersistenceService extends
        org.ow2.petals.clientserverapi.system.persistence.PersistenceService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.System.PersistenceService";

    /**
     * The default URL configuration (File database)
     */
    static final String DEFAULT_URL_PREFIX = "jdbc:hsqldb:file:";

    static final String DEFAULT_URL_SUFFIX = "persistenceService;shutdown=true";

    /**
     * Add a Message Exchange to the persistence service.
     * 
     * @param exchange
     *            the exchange
     * @param storeExchange
     *            if {@code true}, persist the body of the exchange. If
     *            {@code false}, persist only the exchangeId and the current
     *            timestamp
     * @param isSend
     *            if {@code true}, it is an exchange sent<br>
     *            if {@code false}, it is an exchange receipt if {@code null},
     *            it is nor a send or receipt
     * @throws SQLException
     * @throws IOException
     */
    void addMessageExchange(PetalsMessageExchange exchange, boolean storeExchange, Boolean isSent)
            throws SQLException, IOException;

    /**
     * Clear the exchange storage.
     */
    void clearExchangeStorage();

    /**
     * Delete the exchange.
     * 
     * @param exchangeId
     */
    void deleteMessageExchange(String exchangeId) throws SQLException;

    /**
     * Get the exchange storage duration. When the duration is reached, the
     * exchanges are removed from the Persistence Service.
     * 
     * @param duration
     */
    long getExchangeStorageDuration();

    /**
     * Retrieve a Message Exchange from the persistence.
     * 
     * @param exchangeId
     *            the Id of the exchange
     * 
     * @return the recreated {@code PetalsMessageExchange} instance
     * 
     * @throws SQLException
     * @throws IOException
     * @throws ClassNotFoundException
     */
    PetalsMessageExchange getMessageExchange(String exchangeId) throws SQLException, IOException,
            ClassNotFoundException;

    /**
     * Get the Message Exchange sent/received history from the persistence.
     * 
     * @param exchangeId
     *            the Id of the exchange
     * 
     * @return a map of events, the type (sent or received) as key, the
     *         timestamp in millisecond as value.
     * 
     * @throws SQLException
     */
    Map<String, Long> getMessageExchangeHistory(String exchangeId) throws SQLException;

    /**
     * Retrieve a list of Message Exchange Ids from the persistence matching the
     * given criteria and the exchange persistence duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * @param startDate
     *            the start date
     * @param endDate
     *            the end date
     * 
     * @return the {@code List} of Exchange Ids.
     * @throws SQLException
     */
    List<String> getMessageExchangeIds(QName interfaceName, QName serviceName, String endpointName,
            QName operationName, long startDate, long endDate) throws SQLException;

    /**
     * Retrieve a list of Message Exchange Ids from the persistence matching the
     * given the given exchange property and the exchange persistence duration
     * 
     ** @param propertyKey
     *            The name of the property
     * @param propertyValue
     *            The value of the property as a String
     * @return the list of the exchange Ids
     * 
     * @return the {@code List} of Exchange Ids.
     * @throws SQLException
     * @throws IOException
     */
    List<String> getMessageExchangeIds(String propertyKey, String propertyValue)
            throws SQLException, IOException;

    /**
     * Count the number of processed exchanges matching the given criteria.
     * 
     * @param exchangeId
     *            the Id of the exchange
     */
    int getMessageExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName, long startDate, long endDate) throws SQLException;

    /**
     * Set the exchange storage duration. When the duration is reached, the
     * exchanges are removed from the Persistence Service.
     * 
     * @param duration
     */
    void setExchangeStorageDuration(long duration);

    /**
     * Update a Message Exchange in the persistence service.
     * 
     * @param exchange
     *            the exchange
     * @param storeExchange
     *            if {@code true}, persist the body of the exchange. If
     *            {@code false}, persist only the exchangeId and the current
     *            timestamp
     * @param isSend
     *            if {@code true}, it is an exchange sent<br>
     *            if {@code false}, it is an exchange receipt if {@code null},
     *            it is nor a send or receipt
     * @throws SQLException
     * @throws IOException
     */
    void updateMessageExchange(PetalsMessageExchange exchange, boolean storeExchange, Boolean isSent)
            throws SQLException, IOException;

}
