/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.net.URI;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.clientserverapi.topology.TopologyService.DomainPropertyName;

/**
 * Unit tests of {@link DomainConfiguration}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DomainConfigurationTest {

    private final static String DOMAIN_NAME = "domain-name-test";

    private final static String DOMAIN_DESCRIPTION = "domain description test";

    private final static String JNDI_FACTORY = "jndi.factory";

    private final static URI JNDI_PROVIDER_URL = URI.create("jndi.provider/url");

    private final static String JNDI_SECURITY_PRINCIPAL = "jndi-security-principal";

    private final static String JNDI_SECURITY_CREDENTIALS = "jndiSecurityCredentials";

    private final static int JNDI_BATCH_SIZE = 10;

    private final static int JNDI_POOL_SIZE = 20;

    private DomainConfiguration domainConfig;

    @Before
    public void before() {
        this.domainConfig = new DomainConfiguration();
        this.domainConfig.setName(DOMAIN_NAME);
        this.domainConfig.setDescription(DOMAIN_DESCRIPTION);
        final JndiConfiguration jndiConfiguration = new JndiConfiguration();
        jndiConfiguration.setJndiFactory(JNDI_FACTORY);
        jndiConfiguration.setJndiProviderUrl(JNDI_PROVIDER_URL);
        jndiConfiguration.setJndiSecurityPrincipal(JNDI_SECURITY_PRINCIPAL);
        jndiConfiguration.setJndiSecurityCredentials(JNDI_SECURITY_CREDENTIALS);
        jndiConfiguration.setJndiBatchSize(JNDI_BATCH_SIZE);
        jndiConfiguration.setJndiPoolSize(JNDI_POOL_SIZE);
        this.domainConfig.setJndiConfiguration(jndiConfiguration);
    }

    /**
     * <p>
     * Check the conversion of a {@link DomainConfiguration} into a {@link Map}, where:
     * <ul>
     * <li>all fields are set to a value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_AllValuesSet() {
        final Map<String, String> domainConfigMap = this.domainConfig.toMap();

        this.assertBaseMap(domainConfigMap, DOMAIN_NAME, DOMAIN_DESCRIPTION, JNDI_FACTORY,
                JNDI_PROVIDER_URL.toString(), JNDI_SECURITY_PRINCIPAL, JNDI_SECURITY_CREDENTIALS,
                Integer.toString(JNDI_BATCH_SIZE), Integer.toString(JNDI_POOL_SIZE));
    }

    /**
     * <p>
     * Check the conversion of a {@link DomainConfiguration} into a {@link Map}, where:
     * <ul>
     * <li>all fields are set to a value,</li>
     * <li>except the description that is unset</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithoutDescription() {
        this.domainConfig.setDescription(null);
        final Map<String, String> domainConfigMap = this.domainConfig.toMap();

        this.assertBaseMap(domainConfigMap, DOMAIN_NAME, "", JNDI_FACTORY, JNDI_PROVIDER_URL.toString(),
                JNDI_SECURITY_PRINCIPAL, JNDI_SECURITY_CREDENTIALS, Integer.toString(JNDI_BATCH_SIZE),
                Integer.toString(JNDI_POOL_SIZE));
    }

    /**
     * <p>
     * Check the conversion of a {@link DomainConfiguration} into a {@link Map}, where:
     * <ul>
     * <li>all fields are set to a value,</li>
     * <li>except the JNDI configuration fields that are unset</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithoutJndi() {
        this.domainConfig.setJndiConfiguration(null);
        final Map<String, String> domainConfigMap = this.domainConfig.toMap();

        this.assertBaseMap(domainConfigMap, DOMAIN_NAME, DOMAIN_DESCRIPTION, "", "", "", "", "", "");
    }

    /**
     * <p>
     * Check the conversion of a {@link DomainConfiguration} into a {@link Map}, where:
     * <ul>
     * <li>all fields are set to a value,</li>
     * <li>except the JNDI factory that is unset</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithoutJndiFactory() {
        this.domainConfig.getJndiConfiguration().setJndiFactory(null);
        final Map<String, String> domainConfigMap = this.domainConfig.toMap();

        this.assertBaseMap(domainConfigMap, DOMAIN_NAME, DOMAIN_DESCRIPTION, "", JNDI_PROVIDER_URL.toString(),
                JNDI_SECURITY_PRINCIPAL, JNDI_SECURITY_CREDENTIALS, Integer.toString(JNDI_BATCH_SIZE),
                Integer.toString(JNDI_POOL_SIZE));
    }

    /**
     * <p>
     * Check the conversion of a {@link DomainConfiguration} into a {@link Map}, where:
     * <ul>
     * <li>all fields are set to a value,</li>
     * <li>except the JNDI providerUrl that is unset</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithoutJndiProviderURL() {
        this.domainConfig.getJndiConfiguration().setJndiProviderUrl(null);
        final Map<String, String> domainConfigMap = this.domainConfig.toMap();

        this.assertBaseMap(domainConfigMap, DOMAIN_NAME, DOMAIN_DESCRIPTION, JNDI_FACTORY, "", JNDI_SECURITY_PRINCIPAL,
                JNDI_SECURITY_CREDENTIALS, Integer.toString(JNDI_BATCH_SIZE), Integer.toString(JNDI_POOL_SIZE));
    }

    private void assertBaseMap(final Map<String, String> domainConfigMap, final String domainName,
            final String domainDescr, final String jndiFactory, final String jndiProviderUrl,
            final String jndiSecurityPrincipal, final String jndiSecurityCredentials, final String jndiBatchSize,
            final String jndiPoolSize) {

        assertEquals("Unexpected domain name", domainName, domainConfigMap.get(DomainPropertyName.CONF_DOMAIN_NAME));
        if (domainDescr != null) {
            assertEquals("Unexpected domain description", domainDescr,
                    domainConfigMap.get(DomainPropertyName.CONF_DOMAIN_DESCRIPTION));
        } else {
            assertNull("Domain description returned", domainConfigMap.get(DomainPropertyName.CONF_DOMAIN_DESCRIPTION));
        }
        assertEquals("Unexpected domain JNDI factory", jndiFactory,
                domainConfigMap.get(DomainPropertyName.CONF_JNDI_FACTORY));
        if (jndiProviderUrl != null) {
            assertEquals("Unexpected domain JNDI provider URL", jndiProviderUrl,
                    domainConfigMap.get(DomainPropertyName.CONF_JNDI_PROVIDER_URL));
        } else {
            assertNull("Domain  JNDI provider URL returned",
                    domainConfigMap.get(DomainPropertyName.CONF_JNDI_PROVIDER_URL));
        }
        assertEquals("Unexpected domain JNDI security principal", jndiSecurityPrincipal,
                domainConfigMap.get(DomainPropertyName.CONF_JNDI_SECURITY_PRINCIPAL));
        assertEquals("Unexpected domain JNDI security credentials", jndiSecurityCredentials,
                domainConfigMap.get(DomainPropertyName.CONF_JNDI_SECURITY_CREDENTIALS));
        assertEquals("Unexpected domain JNDI batch size", jndiBatchSize,
                domainConfigMap.get(DomainPropertyName.CONF_JNDI_BATCH_SIZE));
        assertEquals("Unexpected domain JNDI pool size", jndiPoolSize,
                domainConfigMap.get(DomainPropertyName.CONF_JNDI_POOL_SIZE));
    }

}
